from datetime import datetime
from typing import Any
from uuid import UUID

from pydantic import BaseModel, Field, field_serializer


class AuthzResponse(BaseModel):
    """Schema for a response from OPA"""

    decision_id: UUID = Field(
        ...,
        description="Decision ID for for the specific decision",
        examples=["8851dce0-7546-4e81-a89d-111cbec376c1"],
    )
    result: bool = Field(..., description="Result of the Authz request")

    @field_serializer("decision_id")
    def serialize_decision_id(self, decision_id: UUID, _info: Any) -> str:
        return str(decision_id)


class AuthzRequest(BaseModel):
    """Schema for a Request to OPA"""

    uid: str = Field(..., description="lifescience id of user", examples=["28c5353b8bb34984a8bd4169ba94c606"])
    operation: str = Field(..., description="Operation the user wants to perform", examples=["read"])
    resource: str = Field(..., description="Resource the operation should be performed on", examples=["bucket"])


class JWT(BaseModel):
    """
    Schema for a JWT. Only for convenience
    """

    exp: datetime
    sub: str
    raw_token: str


class ErrorDetail(BaseModel):
    """
    Schema for a error due to a rejected request.
    """

    detail: str = Field(..., description="Detail about the occurred error")
