from uuid import uuid4

import pytest
from sqlalchemy.ext.asyncio import AsyncSession

from app.crud.crud_user import CRUDUser
from app.tests.utils.user import UserWithAuthHeader


class TestUserCRUD:
    @pytest.mark.asyncio
    async def test_get_user_by_id(self, db: AsyncSession, random_user: UserWithAuthHeader) -> None:
        """
        Test for getting a user by id from the User CRUD Repository.

        Parameters
        ----------
        db : sqlalchemy.ext.asyncio.AsyncSession.
            Async database session to perform query on. pytest fixture.
        random_user : clowmdb.models.User
            Random user for testing. pytest fixture.
        """
        user = await CRUDUser.get(db, random_user.user.uid)
        assert user
        assert random_user.user.uid == user.uid
        assert random_user.user.display_name == user.display_name

    @pytest.mark.asyncio
    async def test_get_unknown_user_by_id(
        self,
        db: AsyncSession,
    ) -> None:
        """
        Test for getting an unknown user by id from the User CRUD Repository.

        Parameters
        ----------
        db : sqlalchemy.ext.asyncio.AsyncSession.
            Async database session to perform query on. pytest fixture.
        """
        user = await CRUDUser.get(db, uuid4())
        assert user is None
